@@ -122,7 +122,7 @@ boxed_resolve(JSContext       *context,
               bool            *resolved)
 {
     Boxed *priv;
-    char *name = NULL;
+    GjsAutoJSChar name(context);
 
     if (!gjs_get_string_id(context, id, &name)) {
         *resolved = false;
@@ -133,10 +133,8 @@ boxed_resolve(JSContext       *context,
     gjs_debug_jsprop(GJS_DEBUG_GBOXED, "Resolve prop '%s' hook obj %p priv %p",
                      name, obj.get(), priv);
 
-    if (priv == NULL) {
-        g_free(name);
+    if (priv == nullptr)
         return false; /* wrong class */
-    }
 
     if (priv->gboxed == NULL) {
         /* We are the prototype, so look for methods and other class properties */
@@ -164,7 +162,6 @@ boxed_resolve(JSContext       *context,
                 if (gjs_define_function(context, obj, priv->gtype,
                                         (GICallableInfo *)method_info) == NULL) {
                     g_base_info_unref( (GIBaseInfo*) method_info);
-                    g_free(name);
                     return false;
                 }
 
@@ -185,7 +182,6 @@ boxed_resolve(JSContext       *context,
          */
         *resolved = false;
     }
-    g_free(name);
     return true;
 }
 
@@ -283,7 +279,7 @@ boxed_init_from_props(JSContext   *context,
     JS::RootedId prop_id(context);
     for (ix = 0, length = ids.length(); ix < length; ix++) {
         GIFieldInfo *field_info;
-        char *name = NULL;
+        GjsAutoJSChar name(context);
 
         if (!gjs_get_string_id(context, ids[ix], &name))
             return false;
@@ -291,8 +287,7 @@ boxed_init_from_props(JSContext   *context,
         field_info = (GIFieldInfo *) g_hash_table_lookup(priv->field_map, name);
         if (field_info == NULL) {
             gjs_throw(context, "No field %s on boxed type %s",
-                      name, g_base_info_get_name((GIBaseInfo *)priv->info));
-            g_free(name);
+                      name.get(), g_base_info_get_name((GIBaseInfo *)priv->info));
             return false;
         }
 
@@ -300,17 +295,11 @@ boxed_init_from_props(JSContext   *context,
          * doesn't know that */
         prop_id = ids[ix];
         if (!gjs_object_require_property(context, props, "property list",
-                                         prop_id, &value)) {
-            g_free(name);
+                                         prop_id, &value))
             return false;
-        }
 
-        if (!boxed_set_field_from_value(context, priv, field_info, value)) {
-            g_free(name);
+        if (!boxed_set_field_from_value(context, priv, field_info, value))
             return false;
-        }
-
-        g_free(name);
     }
 
     return true;
