@@ -112,7 +112,7 @@ interface_resolve(JSContext       *context,
                   bool            *resolved)
 {
     Interface *priv;
-    char *name = NULL;
+    GjsAutoJSChar name(context);
     GIFunctionInfo *method_info;
 
     if (!gjs_get_string_id(context, id, &name)) {
@@ -122,17 +122,14 @@ interface_resolve(JSContext       *context,
 
     priv = priv_from_js(context, obj);
 
-    if (priv == NULL) {
-        g_free(name);
+    if (priv == nullptr)
         return false;
-    }
 
     /* If we have no GIRepository information then this interface was defined
      * from within GJS. In that case, it has no properties that need to be
      * resolved from within C code, as interfaces cannot inherit. */
     if (priv->info == NULL) {
         *resolved = false;
-        g_free(name);
         return true;
     }
 
@@ -144,7 +141,6 @@ interface_resolve(JSContext       *context,
                                     priv->gtype,
                                     (GICallableInfo*)method_info) == NULL) {
                 g_base_info_unref((GIBaseInfo*)method_info);
-                g_free(name);
                 return false;
             }
 
@@ -158,7 +154,6 @@ interface_resolve(JSContext       *context,
         *resolved = false;
     }
 
-    g_free(name);
     return true;
 }
 
