@@ -26,6 +26,7 @@
 #include <glib.h>
 #include "byteArray.h"
 #include "gi/boxed.h"
+#include "jsapi-class.h"
 #include "jsapi-wrapper.h"
 #include "jsapi-util-args.h"
 #include <girepository.h>
@@ -52,6 +53,7 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(byte_array);
 static void   byte_array_finalize      (JSFreeOp     *fop,
                                         JSObject     *obj);
 
+static JSObject *gjs_byte_array_get_proto(JSContext *);
 
 struct JSClass gjs_byte_array_class = {
     "ByteArray",
@@ -403,7 +405,7 @@ to_string_func(JSContext *context,
                JS::Value *vp)
 {
     GJS_GET_PRIV(context, argc, vp, argv, to, ByteArrayInstance, priv);
-    char *encoding;
+    GjsAutoJSChar encoding(context);
     bool encoding_is_utf8;
     gchar *data;
 
@@ -420,13 +422,7 @@ to_string_func(JSContext *context,
          * doesn't matter much though. encoding_is_utf8 is
          * just an optimization anyway.
          */
-        if (strcmp(encoding, "UTF-8") == 0) {
-            encoding_is_utf8 = true;
-            g_free(encoding);
-            encoding = NULL;
-        } else {
-            encoding_is_utf8 = false;
-        }
+        encoding_is_utf8 = (strcmp(encoding, "UTF-8") == 0);
     } else {
         encoding_is_utf8 = true;
     }
@@ -459,7 +455,6 @@ to_string_func(JSContext *context,
                            NULL, /* bytes read */
                            &bytes_written,
                            &error);
-        g_free(encoding);
         if (u16_str == NULL) {
             /* frees the GError */
             gjs_throw_g_error(context, error);
@@ -507,30 +502,12 @@ to_gbytes_func(JSContext *context,
     return true;
 }
 
-/* Ensure that the module and class objects exists, and that in turn
- * ensures that JS_InitClass has been called. */
-static JSObject *
-byte_array_get_prototype(JSContext *context)
-{
-    JS::RootedValue retval(context,
-        gjs_get_global_slot(context, GJS_GLOBAL_SLOT_BYTE_ARRAY_PROTOTYPE));
-
-    if (!retval.isObject()) {
-        if (!gjs_eval_with_scope(context, JS::NullPtr(),
-                                 "imports.byteArray.ByteArray.prototype;", -1,
-                                 "<internal>", &retval))
-            g_error ("Could not import byte array prototype\n");
-    }
-
-    return &retval.toObject();
-}
-
 static JSObject*
 byte_array_new(JSContext *context)
 {
     ByteArrayInstance *priv;
 
-    JS::RootedObject proto(context, byte_array_get_prototype(context));
+    JS::RootedObject proto(context, gjs_byte_array_get_proto(context));
     JS::RootedObject array(context,
         JS_NewObjectWithGivenProto(context, &gjs_byte_array_class, proto));
 
@@ -550,7 +527,7 @@ from_string_func(JSContext *context,
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
     ByteArrayInstance *priv;
-    char *encoding;
+    GjsAutoJSChar encoding(context);
     bool encoding_is_utf8;
     JS::RootedObject obj(context, byte_array_new(context));
 
@@ -578,13 +555,7 @@ from_string_func(JSContext *context,
          * doesn't matter much though. encoding_is_utf8 is
          * just an optimization anyway.
          */
-        if (strcmp(encoding, "UTF-8") == 0) {
-            encoding_is_utf8 = true;
-            g_free(encoding);
-            encoding = NULL;
-        } else {
-            encoding_is_utf8 = false;
-        }
+        encoding_is_utf8 = (strcmp(encoding, "UTF-8") == 0);
     } else {
         encoding_is_utf8 = true;
     }
@@ -593,15 +564,16 @@ from_string_func(JSContext *context,
         /* optimization? avoids iconv overhead and runs
          * libmozjs hardwired utf16-to-utf8.
          */
-        char *utf8 = NULL;
+        GjsAutoJSChar utf8(context);
         if (!gjs_string_to_utf8(context,
                                 argv[0],
                                 &utf8))
             return false;
 
         g_byte_array_set_size(priv->array, 0);
-        g_byte_array_append(priv->array, (guint8*) utf8, strlen(utf8));
-        g_free(utf8);
+        g_byte_array_append(priv->array,
+                            reinterpret_cast<const guint8*>(utf8.get()),
+                            strlen(utf8));
     } else {
         JSString *str = argv[0].toString();  /* Rooted by argv */
         GError *error = NULL;
@@ -639,7 +611,6 @@ from_string_func(JSContext *context,
             }
         }
 
-        g_free(encoding);
         if (encoded == NULL) {
             /* frees the GError */
             gjs_throw_g_error(context, error);
@@ -758,7 +729,7 @@ gjs_byte_array_from_byte_array (JSContext *context,
     g_return_val_if_fail(context != NULL, NULL);
     g_return_val_if_fail(array != NULL, NULL);
 
-    JS::RootedObject proto(context, byte_array_get_prototype(context));
+    JS::RootedObject proto(context, gjs_byte_array_get_proto(context));
     JS::RootedObject object(context,
         JS_NewObjectWithGivenProto(context, &gjs_byte_array_class, proto));
 
@@ -823,18 +794,20 @@ gjs_byte_array_peek_data (JSContext       *context,
     }
 }
 
-JSPropertySpec gjs_byte_array_proto_props[] = {
+static JSPropertySpec gjs_byte_array_proto_props[] = {
     JS_PSGS("length", byte_array_length_getter, byte_array_length_setter,
             JSPROP_PERMANENT),
     JS_PS_END
 };
 
-JSFunctionSpec gjs_byte_array_proto_funcs[] = {
+static JSFunctionSpec gjs_byte_array_proto_funcs[] = {
     JS_FS("toString", to_string_func, 0, 0),
     JS_FS("toGBytes", to_gbytes_func, 0, 0),
     JS_FS_END
 };
 
+static JSFunctionSpec *gjs_byte_array_static_funcs = nullptr;
+
 static JSFunctionSpec gjs_byte_array_module_funcs[] = {
     JS_FS("fromString", from_string_func, 1, 0),
     JS_FS("fromArray", from_array_func, 1, 0),
@@ -842,29 +815,15 @@ static JSFunctionSpec gjs_byte_array_module_funcs[] = {
     JS_FS_END
 };
 
+GJS_DEFINE_PROTO_FUNCS(byte_array)
+
 bool
-gjs_define_byte_array_stuff(JSContext              *context,
+gjs_define_byte_array_stuff(JSContext              *cx,
                             JS::MutableHandleObject module)
 {
-    JSObject *prototype;
+    module.set(JS_NewPlainObject(cx));
 
-    module.set(JS_NewObject(context, NULL));
-
-    prototype = JS_InitClass(context, module, JS::NullPtr(),
-                             &gjs_byte_array_class,
-                             gjs_byte_array_constructor,
-                             0,
-                             &gjs_byte_array_proto_props[0],
-                             &gjs_byte_array_proto_funcs[0],
-                             NULL,
-                             NULL);
-
-    if (!JS_DefineFunctions(context, module, &gjs_byte_array_module_funcs[0]))
-        return false;
-
-    g_assert(gjs_get_global_slot(context, GJS_GLOBAL_SLOT_BYTE_ARRAY_PROTOTYPE).isUndefined());
-    gjs_set_global_slot(context, GJS_GLOBAL_SLOT_BYTE_ARRAY_PROTOTYPE,
-                        JS::ObjectOrNullValue(prototype));
-
-    return true;
+    JS::RootedObject proto(cx);
+    return gjs_byte_array_define_proto(cx, module, &proto) &&
+        JS_DefineFunctions(cx, module, gjs_byte_array_module_funcs);
 }
