@@ -29,6 +29,7 @@
 #include <util/misc.h>
 #include <util/error.h>
 
+#include "jsapi-class.h"
 #include "jsapi-util.h"
 #include "jsapi-wrapper.h"
 #include "context-private.h"
@@ -210,17 +211,15 @@ throw_property_lookup_error(JSContext       *cx,
     /* remember gjs_throw() is a no-op if JS_GetProperty()
      * already set an exception
      */
-    char *name;
+    GjsAutoJSChar name(cx);
     gjs_get_string_id(cx, property_name, &name);
 
     if (description)
-        gjs_throw(cx, "No property '%s' in %s (or %s)", name, description,
+        gjs_throw(cx, "No property '%s' in %s (or %s)", name.get(), description,
                   reason);
     else
-        gjs_throw(cx, "No property '%s' in object %p (or %s)", name,
+        gjs_throw(cx, "No property '%s' in object %p (or %s)", name.get(),
                   obj.address(), reason);
-
-    g_free(name);
 }
 
 /* Returns whether the object had the property; if the object did
@@ -301,8 +300,10 @@ gjs_object_require_property(JSContext       *cx,
                             char           **value)
 {
     JS::RootedValue prop_value(cx);
+    GjsAutoJSChar value_tmp(cx);
     if (JS_GetPropertyById(cx, obj, property_name, &prop_value) &&
-        gjs_string_to_utf8(cx, prop_value, value)) {
+        gjs_string_to_utf8(cx, prop_value, &value_tmp)) {
+        *value = value_tmp.copy();
         return true;
     }
 
@@ -431,7 +432,7 @@ gjs_string_readable (JSContext   *context,
                      JSString    *string)
 {
     GString *buf = g_string_new("");
-    char *chars;
+    GjsAutoJSChar chars(context);
 
     JS_BeginRequest(context);
 
@@ -453,7 +454,6 @@ gjs_string_readable (JSContext   *context,
         g_free(escaped);
     } else {
         g_string_append(buf, chars);
-        g_free(chars);
     }
 
     g_string_append_c(buf, '"');
@@ -564,11 +564,11 @@ static char *
 utf8_exception_from_non_gerror_value(JSContext      *cx,
                                      JS::HandleValue exc)
 {
-    char *utf8_exception = NULL;
+    GjsAutoJSChar utf8_exception(cx);
     JS::RootedString exc_str(cx, JS::ToString(cx, exc));
     if (exc_str != NULL)
         gjs_string_to_utf8(cx, JS::StringValue(exc_str), &utf8_exception);
-    return utf8_exception;
+    return utf8_exception.copy();
 }
 
 bool
@@ -576,7 +576,8 @@ gjs_log_exception_full(JSContext       *context,
                        JS::HandleValue  exc,
                        JS::HandleString message)
 {
-    char *utf8_exception, *utf8_message;
+    char *utf8_exception;
+    GjsAutoJSChar utf8_message(context);
     bool is_syntax;
 
     JS_BeginRequest(context);
@@ -595,7 +596,7 @@ gjs_log_exception_full(JSContext       *context,
                                              gerror->message);
         } else {
             JS::RootedValue js_name(context);
-            char *utf8_name;
+            GjsAutoJSChar utf8_name(context);
 
             if (gjs_object_get_property(context, exc_obj,
                                         GJS_STRING_NAME, &js_name) &&
@@ -610,8 +611,6 @@ gjs_log_exception_full(JSContext       *context,
 
     if (message != NULL)
         gjs_string_to_utf8(context, JS::StringValue(message), &utf8_message);
-    else
-        utf8_message = NULL;
 
     /* We log syntax errors differently, because the stack for those includes
        only the referencing module, but we want to print out the filename and
@@ -621,7 +620,7 @@ gjs_log_exception_full(JSContext       *context,
     if (is_syntax) {
         JS::RootedValue js_lineNumber(context), js_fileName(context);
         unsigned lineNumber;
-        char *utf8_fileName;
+        GjsAutoJSChar utf8_filename(context);
 
         gjs_object_get_property(context, exc_obj, GJS_STRING_LINE_NUMBER,
                                 &js_lineNumber);
@@ -629,50 +628,47 @@ gjs_log_exception_full(JSContext       *context,
                                 &js_fileName);
 
         if (js_fileName.isString())
-            gjs_string_to_utf8(context, js_fileName, &utf8_fileName);
+            gjs_string_to_utf8(context, js_fileName, &utf8_filename);
         else
-            utf8_fileName = g_strdup("unknown");
+            utf8_filename.reset(context, JS_strdup(context, "unknown"));
 
         lineNumber = js_lineNumber.toInt32();
 
-        if (utf8_message) {
-            g_critical("JS ERROR: %s: %s @ %s:%u", utf8_message, utf8_exception,
-                       utf8_fileName, lineNumber);
+        if (message != NULL) {
+            g_critical("JS ERROR: %s: %s @ %s:%u", utf8_message.get(), utf8_exception,
+                       utf8_filename.get(), lineNumber);
         } else {
             g_critical("JS ERROR: %s @ %s:%u", utf8_exception,
-                       utf8_fileName, lineNumber);
+                       utf8_filename.get(), lineNumber);
         }
 
-        g_free(utf8_fileName);
     } else {
-        char *utf8_stack;
+        GjsAutoJSChar utf8_stack(context);
         JS::RootedValue stack(context);
+        bool have_utf8_stack = false;
 
         if (exc.isObject() &&
             gjs_object_get_property(context, exc_obj, GJS_STRING_STACK,
                                     &stack) &&
-            stack.isString())
+            stack.isString()) {
             gjs_string_to_utf8(context, stack, &utf8_stack);
-        else
-            utf8_stack = NULL;
+            have_utf8_stack = true;
+        }
 
-        if (utf8_message) {
-            if (utf8_stack)
-                g_warning("JS ERROR: %s: %s\n%s", utf8_message, utf8_exception, utf8_stack);
+        if (message != nullptr) {
+            if (have_utf8_stack)
+                g_warning("JS ERROR: %s: %s\n%s", utf8_message.get(), utf8_exception, utf8_stack.get());
             else
-                g_warning("JS ERROR: %s: %s", utf8_message, utf8_exception);
+                g_warning("JS ERROR: %s: %s", utf8_message.get(), utf8_exception);
         } else {
-            if (utf8_stack)
-                g_warning("JS ERROR: %s\n%s", utf8_exception, utf8_stack);
+            if (have_utf8_stack)
+                g_warning("JS ERROR: %s\n%s", utf8_exception, utf8_stack.get());
             else
                 g_warning("JS ERROR: %s", utf8_exception);
         }
-
-        g_free(utf8_stack);
     }
 
     g_free(utf8_exception);
-    g_free(utf8_message);
 
     JS_EndRequest(context);
 
@@ -866,7 +862,7 @@ gjs_schedule_gc_if_needed (JSContext *context)
  */
 const char *
 gjs_strip_unix_shebang(const char  *script,
-                       gssize      *script_len,
+                       size_t      *script_len,
                        int         *start_line_number_out)
 {
     g_assert(script_len);
@@ -912,12 +908,13 @@ gjs_eval_with_scope(JSContext             *context,
 {
     int start_line_number = 1;
     JSAutoRequest ar(context);
+    size_t real_len = script_len;
 
     if (script_len < 0)
-        script_len = strlen(script);
+        real_len = strlen(script);
 
     script = gjs_strip_unix_shebang(script,
-                                    &script_len,
+                                    &real_len,
                                     &start_line_number);
 
     /* log and clear exception if it's set (should not be, normally...) */
@@ -928,14 +925,20 @@ gjs_eval_with_scope(JSContext             *context,
 
     JS::RootedObject eval_obj(context, object);
     if (!eval_obj)
-        eval_obj = JS_NewObject(context, NULL);
+        eval_obj = JS_NewPlainObject(context);
 
     JS::CompileOptions options(context);
     options.setUTF8(true)
            .setFileAndLine(filename, start_line_number)
            .setSourceIsLazy(true);
 
-    if (!JS::Evaluate(context, eval_obj, options, script, script_len, retval))
+    JS::RootedScript compiled_script(context);
+    if (!JS::Compile(context, object, options, script, real_len, &compiled_script))
+        return false;
+
+    JS::AutoObjectVector scope_chain(context);
+    scope_chain.append(eval_obj);
+    if (!JS_ExecuteScript(context, scope_chain, compiled_script, retval))
         return false;
 
     gjs_schedule_gc_if_needed(context);
