@@ -1,4 +1,4 @@
-/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
+/* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /*
  * Copyright (c) 2009  litl, LLC
  * Copyright (c) 2010  Red Hat, Inc.
@@ -25,13 +25,16 @@
 #ifndef GJS_JSAPI_WRAPPER_H
 #define GJS_JSAPI_WRAPPER_H
 
-#include <js-config.h>  /* SpiderMonkey's #defines that affect public API */
+#include <config.h>
+
 /* COMPAT: SpiderMonkey headers in some places use DEBUG instead of JS_DEBUG */
 /* https://bugzilla.mozilla.org/show_bug.cgi?id=1261161 */
-#if defined(JS_DEBUG) && JS_DEBUG
+#ifdef HAVE_DEBUG_SPIDERMONKEY
 #define DEBUG 1
 #endif
 
+#include <js-config.h>  /* SpiderMonkey's #defines that affect public API */
+
 #if defined(__clang__) || __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6)
 #pragma GCC system_header
 #endif
