@@ -57,7 +57,7 @@ gjs_locale_to_upper_case (JSContext *context,
                           JS::MutableHandleValue retval)
 {
     bool success = false;
-    char *utf8 = NULL;
+    GjsAutoJSChar utf8(context);
     char *upper_case_utf8 = NULL;
 
     if (!gjs_string_to_utf8(context, JS::StringValue(src), &utf8))
@@ -71,7 +71,6 @@ gjs_locale_to_upper_case (JSContext *context,
     success = true;
 
 out:
-    g_free(utf8);
     g_free(upper_case_utf8);
 
     return success;
@@ -83,7 +82,7 @@ gjs_locale_to_lower_case (JSContext *context,
                           JS::MutableHandleValue retval)
 {
     bool success = false;
-    char *utf8 = NULL;
+    GjsAutoJSChar utf8(context);
     char *lower_case_utf8 = NULL;
 
     if (!gjs_string_to_utf8(context, JS::StringValue(src), &utf8))
@@ -97,7 +96,6 @@ gjs_locale_to_lower_case (JSContext *context,
     success = true;
 
 out:
-    g_free(utf8);
     g_free(lower_case_utf8);
 
     return success;
@@ -109,22 +107,15 @@ gjs_locale_compare (JSContext *context,
                     JS::HandleString src_2,
                     JS::MutableHandleValue retval)
 {
-    bool success = false;
-    char *utf8_1 = NULL, *utf8_2 = NULL;
+    GjsAutoJSChar utf8_1(context), utf8_2(context);
 
     if (!gjs_string_to_utf8(context, JS::StringValue(src_1), &utf8_1) ||
         !gjs_string_to_utf8(context, JS::StringValue(src_2), &utf8_2))
-        goto out;
+        return false;
 
     retval.setInt32(g_utf8_collate(utf8_1, utf8_2));
 
-    success = true;
-
-out:
-    g_free(utf8_1);
-    g_free(utf8_2);
-
-    return success;
+    return true;
 }
 
 static bool
@@ -299,8 +290,22 @@ gjs_runtime_for_current_thread(void)
         data = g_new0(RuntimeData, 1);
         JS_SetRuntimePrivate(runtime, data);
 
+        // commented are defaults in moz-24
         JS_SetNativeStackQuota(runtime, 1024*1024);
-        JS_SetGCParameter(runtime, JSGC_MAX_BYTES, 0xffffffff);
+        JS_SetGCParameter(runtime, JSGC_MAX_MALLOC_BYTES, 128*1024*1024);
+        JS_SetGCParameter(runtime, JSGC_MAX_BYTES, -1);
+        JS_SetGCParameter(runtime, JSGC_MODE, JSGC_MODE_INCREMENTAL);
+        JS_SetGCParameter(runtime, JSGC_SLICE_TIME_BUDGET, 10); /* ms */
+        // JS_SetGCParameter(runtime, JSGC_HIGH_FREQUENCY_TIME_LIMIT, 1000); /* ms */
+        JS_SetGCParameter(runtime, JSGC_DYNAMIC_MARK_SLICE, true);
+        JS_SetGCParameter(runtime, JSGC_DYNAMIC_HEAP_GROWTH, true);
+        // JS_SetGCParameter(runtime, JSGC_LOW_FREQUENCY_HEAP_GROWTH, 150);
+        // JS_SetGCParameter(runtime, JSGC_HIGH_FREQUENCY_HEAP_GROWTH_MIN, 150);
+        // JS_SetGCParameter(runtime, JSGC_HIGH_FREQUENCY_HEAP_GROWTH_MAX, 300);
+        // JS_SetGCParameter(runtime, JSGC_HIGH_FREQUENCY_LOW_LIMIT, 100);
+        // JS_SetGCParameter(runtime, JSGC_HIGH_FREQUENCY_HIGH_LIMIT, 500);
+        // JS_SetGCParameter(runtime, JSGC_ALLOCATION_THRESHOLD, 30);
+        // JS_SetGCParameter(runtime, JSGC_DECOMMIT_THRESHOLD, 32);
         JS_SetLocaleCallbacks(runtime, &gjs_locale_callbacks);
         JS_AddFinalizeCallback(runtime, gjs_finalize_callback, data);
         JS_SetErrorReporter(runtime, gjs_error_reporter);
