@@ -1,11 +1,21 @@
 #!/bin/sh
 
 if test "$GJS_USE_UNINSTALLED_FILES" = "1"; then
-    gjs="$TOP_BUILDDIR"/gjs-console
+    gjs="$TOP_BUILDDIR/gjs-console"
 else
-    gjs=gjs-console
+    gjs="gjs-console"
 fi
 
+# This JS script should exit immediately with code 42. If that is not working,
+# then it will exit after 3 seconds as a fallback, with code 0.
+cat <<EOF >exit.js
+const GLib = imports.gi.GLib;
+let loop = GLib.MainLoop.new(null, false);
+GLib.idle_add(GLib.PRIORITY_LOW, () => imports.system.exit(42));
+GLib.timeout_add_seconds(GLib.PRIORITY_HIGH, 3, () => loop.quit());
+loop.run();
+EOF
+
 # this JS script fails if either 1) --help is not passed to it, or 2) the string
 # "sentinel" is not in its search path
 cat <<EOF >help.js
@@ -21,7 +31,7 @@ total=0
 
 report () {
     exit_code=$?
-    total=`expr $total + 1`
+    total=$((total + 1))
     if test $exit_code -eq 0; then
         echo "ok $total - $1"
     else
@@ -31,7 +41,7 @@ report () {
 
 report_xfail () {
     exit_code=$?
-    total=`expr $total + 1`
+    total=$((total + 1))
     if test $exit_code -ne 0; then
         echo "ok $total - $1"
     else
@@ -40,48 +50,54 @@ report_xfail () {
 }
 
 # Test that System.exit() works in gjs-console
-"$gjs" -c 'imports.system.exit(0)' || \
-    fail "System.exit(0) should exit successfully"
-if "gjs" -c 'imports.system.exit(42)' -ne 42; then
-    fail "System.exit(42) should exit with the correct exit code"
-fi
+"$gjs" -c 'imports.system.exit(0)'
+report "System.exit(0) should exit successfully"
+"$gjs" -c 'imports.system.exit(42)'
+test $? -eq 42
+report "System.exit(42) should exit with the correct exit code"
+
+# FIXME: should check -eq 42 specifically, but in debug mode we will be
+# hitting an assertion
+"$gjs" exit.js
+test $? -ne 0
+report "System.exit() should still exit across an FFI boundary"
 
 # gjs --help prints GJS help
 "$gjs" --help >/dev/null
 report "--help should succeed"
-test -n "`"$gjs" --help`"
+test -n "$("$gjs" --help)"
 report "--help should print something"
 
 # print GJS help even if it's not the first argument
 "$gjs" -I . --help >/dev/null
 report "should succeed when --help is not first arg"
-test -n "`"$gjs" -I . --help`"
+test -n "$("$gjs" -I . --help)"
 report "should print something when --help is not first arg"
 
 # --help before a script file name prints GJS help
 "$gjs" --help help.js >/dev/null
 report "--help should succeed before a script file"
-test -n "`"$gjs" --help help.js`"
+test -n "$("$gjs" --help help.js)"
 report "--help should print something before a script file"
 
 # --help before a -c argument prints GJS help
 script='imports.system.exit(1)'
 "$gjs" --help -c "$script" >/dev/null
 report "--help should succeed before -c"
-test -n "`"$gjs" --help -c "$script"`"
+test -n "$("$gjs" --help -c "$script")"
 report "--help should print something before -c"
 
 # --help after a script file name is passed to the script
 "$gjs" -I sentinel help.js --help
 report "--help after script file should be passed to script"
-test -z "`"$gjs" -I sentinel help.js --help`"
+test -z "$("$gjs" -I sentinel help.js --help)"
 report "--help after script file should not print anything"
 
 # --help after a -c argument is passed to the script
 script='if(ARGV[0] !== "--help") imports.system.exit(1)'
 "$gjs" -c "$script" --help
 report "--help after -c should be passed to script"
-test -z "`"$gjs" -c "$script" --help`"
+test -z "$("$gjs" -c "$script" --help)"
 report "--help after -c should not print anything"
 
 # -I after a program is not consumed by GJS
@@ -100,16 +116,16 @@ rm -f foo/coverage.lcov
 # --version works
 "$gjs" --version >/dev/null
 report "--version should work"
-test -n "`"$gjs" --version`"
+test -n "$("$gjs" --version)"
 report "--version should print something"
 
 # --version after a script goes to the script
 script='if(ARGV[0] !== "--version") imports.system.exit(1)'
 "$gjs" -c "$script" --version
 report "--version after -c should be passed to script"
-test -z "`"$gjs" -c "$script" --version`"
+test -z "$("$gjs" -c "$script" --version)"
 report "--version after -c should not print anything"
 
-rm -f help.js
+rm -f exit.js help.js
 
 echo "1..$total"
