@@ -22,4 +22,8 @@ report () {
     grep -q 'addSignalMethods is replacing existing .* connect method'
 report "overwriting method with Signals.addSignalMethods() should warn"
 
+"$gjs" -c 'imports.gi.GLib.get_home_dir("foobar")' 2>&1 | \
+    grep -q 'Too many arguments to .*: expected 0, got 1'
+report "passing too many arguments to a GI function should warn"
+
 echo "1..$total"
