@@ -22,6 +22,7 @@
 
 #include <config.h>
 
+#include "gjs/jsapi-class.h"
 #include "gjs/jsapi-util.h"
 #include "gjs/jsapi-wrapper.h"
 #include <cairo.h>
@@ -33,7 +34,9 @@ typedef struct {
     cairo_path_t    *path;
 } GjsCairoPath;
 
-GJS_DEFINE_PROTO_ABSTRACT("CairoPath", cairo_path, JSCLASS_BACKGROUND_FINALIZE)
+static JSObject *gjs_cairo_path_get_proto(JSContext *);
+
+GJS_DEFINE_PROTO_ABSTRACT("Path", cairo_path, JSCLASS_BACKGROUND_FINALIZE)
 GJS_DEFINE_PRIV_FROM_JS(GjsCairoPath, gjs_cairo_path_class)
 
 static void
@@ -57,6 +60,8 @@ JSFunctionSpec gjs_cairo_path_proto_funcs[] = {
     JS_FS_END
 };
 
+JSFunctionSpec gjs_cairo_path_static_funcs[] = { JS_FS_END };
+
 /**
  * gjs_cairo_path_from_path:
  * @context: the context
@@ -74,8 +79,9 @@ gjs_cairo_path_from_path(JSContext    *context,
     g_return_val_if_fail(context != NULL, NULL);
     g_return_val_if_fail(path != NULL, NULL);
 
+    JS::RootedObject proto(context, gjs_cairo_path_get_proto(context));
     JS::RootedObject object(context,
-        JS_NewObject(context, &gjs_cairo_path_class));
+        JS_NewObjectWithGivenProto(context, &gjs_cairo_path_class, proto));
     if (!object) {
         gjs_throw(context, "failed to create path");
         return NULL;
