@@ -113,6 +113,9 @@ function getSubNodesForNode(node) {
     case 'ComprehensionBlock':
         subNodes.push(node.right);
         break;
+    case 'ComprehensionIf':
+        subNodes.push(node.test);
+        break;
     /* It is very possible that there might be something
      * interesting in the function arguments, so we need to
      * walk them too */
@@ -417,6 +420,8 @@ function _expressionLinesToCounters(expressionLines, nLines) {
         return counters;
 
     for (let i = 1; i < counters.length; i++) {
+        if (!expressionLines.hasOwnProperty(expressionLinesIndex))
+            continue;
         if (expressionLines[expressionLinesIndex] == i) {
             counters[i] = 0;
             expressionLinesIndex++;
@@ -773,12 +778,6 @@ function CoverageStatisticsContainer(prefixes, cache) {
     let coveredFiles = {};
     let cacheMisses = 0;
 
-    function wantsStatisticsFor(filename) {
-        return prefixes.some(function(prefix) {
-            return filename.startsWith(prefix);
-        });
-    }
-
     function createStatisticsFor(filename) {
         let contents = getFileContents(filename);
         let nLines = _getNumberOfLinesForScript(contents);
@@ -799,10 +798,7 @@ function CoverageStatisticsContainer(prefixes, cache) {
     }
 
     function ensureStatisticsFor(filename) {
-        let wantStatistics = wantsStatisticsFor(filename);
-        let haveStatistics = !!coveredFiles[filename];
-
-        if (wantStatistics && !haveStatistics)
+        if (!coveredFiles[filename])
             coveredFiles[filename] = createStatisticsFor(filename);
         return coveredFiles[filename];
     }
